@@ -60,10 +60,11 @@
 #include "qandroideventdispatcher.h"
 #include <android/api-level.h>
 
-#include <QtCore/qresource.h>
-#include <QtCore/qthread.h>
 #include <QtCore/private/qjnihelpers_p.h>
 #include <QtCore/private/qjni_p.h>
+#include <QtCore/qbasicatomic.h>
+#include <QtCore/qresource.h>
+#include <QtCore/qthread.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qhighdpiscaling_p.h>
 
@@ -125,6 +126,8 @@ static const char m_qtTag[] = "Qt";
 static const char m_classErrorMsg[] = "Can't find class \"%s\"";
 static const char m_methodErrorMsg[] = "Can't find method \"%s%s\"";
 
+static QBasicAtomicInt startQtAndroidPluginCalled = Q_BASIC_ATOMIC_INITIALIZER(0);
+
 namespace QtAndroid
 {
     QBasicMutex *platformInterfaceMutex()
@@ -238,6 +241,11 @@ namespace QtAndroid
         QJNIObjectPrivate::callStaticMethod<void>(m_applicationClass, "notifyObjectFocus","(I)V", accessibilityObjectId);
     }
 
+    void notifyQtAndroidPluginRunning(bool running)
+    {
+        QJNIObjectPrivate::callStaticMethod<void>(m_applicationClass, "notifyQtAndroidPluginRunning","(Z)V", running);
+    }
+
     jobject createBitmap(QImage img, JNIEnv *env)
     {
         if (!m_bitmapClass)
@@ -533,7 +541,7 @@ static void waitForServiceSetup(JNIEnv *env, jclass /*clazz*/)
         QtAndroidPrivate::waitForServiceSetup();
 }
 
-static jboolean startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
+static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
 {
     {
         JNIEnv* env = nullptr;
@@ -554,6 +562,7 @@ static jboolean startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
     for (int i = 0; i < m_applicationParams.size(); i++)
         params[i] = static_cast<const char *>(m_applicationParams[i].constData());
 
+    startQtAndroidPluginCalled.fetchAndAddRelease(1);
     int ret = m_main(m_applicationParams.length(), const_cast<char **>(params.data()));
 
     if (m_mainLibraryHnd) {
@@ -572,7 +581,8 @@ static jboolean startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
     sem_destroy(&m_exitSemaphore);
 
     // We must call exit() to ensure that all global objects will be destructed
-    exit(ret);
+    if (!qEnvironmentVariableIsSet("QT_ANDROID_NO_EXIT_CALL"))
+        exit(ret);
 }
 
 static void quitQtCoreApplication(JNIEnv *env, jclass /*clazz*/)
@@ -600,7 +610,9 @@ static void terminateQt(JNIEnv *env, jclass /*clazz*/)
         QAndroidEventDispatcherStopper::instance()->goingToStop(false);
     }
 
-    sem_wait(&m_terminateSemaphore);
+    if (startQtAndroidPluginCalled.loadAcquire())
+        sem_wait(&m_terminateSemaphore);
+
     sem_destroy(&m_terminateSemaphore);
 
     env->DeleteGlobalRef(m_applicationClass);
@@ -643,11 +655,12 @@ static void setDisplayMetrics(JNIEnv */*env*/, jclass /*clazz*/,
                             jint widthPixels, jint heightPixels,
                             jint desktopWidthPixels, jint desktopHeightPixels,
                             jdouble xdpi, jdouble ydpi,
-                            jdouble scaledDensity, jdouble density)
+                            jdouble scaledDensity, jdouble density, bool forceUpdate)
 {
     // Android does not give us the correct screen size for immersive mode, but
     // the surface does have the right size
 
+    bool updateDesktopSize = m_desktopWidthPixels != desktopWidthPixels;
     widthPixels = qMax(widthPixels, desktopWidthPixels);
     heightPixels = qMax(heightPixels, desktopHeightPixels);
 
@@ -668,7 +681,9 @@ static void setDisplayMetrics(JNIEnv */*env*/, jclass /*clazz*/,
         m_androidPlatformIntegration->setDisplayMetrics(qRound(double(widthPixels)  / xdpi * 25.4),
                                                         qRound(double(heightPixels) / ydpi * 25.4));
         m_androidPlatformIntegration->setScreenSize(widthPixels, heightPixels);
-        m_androidPlatformIntegration->setDesktopSize(desktopWidthPixels, desktopHeightPixels);
+        if (updateDesktopSize || forceUpdate) {
+            m_androidPlatformIntegration->setDesktopSize(desktopWidthPixels, desktopHeightPixels);
+        }
     }
 }
 
@@ -794,7 +809,7 @@ static JNINativeMethod methods[] = {
     {"quitQtCoreApplication", "()V", (void *)quitQtCoreApplication},
     {"terminateQt", "()V", (void *)terminateQt},
     {"waitForServiceSetup", "()V", (void *)waitForServiceSetup},
-    {"setDisplayMetrics", "(IIIIDDDD)V", (void *)setDisplayMetrics},
+    {"setDisplayMetrics", "(IIIIDDDDZ)V", (void *)setDisplayMetrics},
     {"setSurface", "(ILjava/lang/Object;II)V", (void *)setSurface},
     {"updateWindow", "()V", (void *)updateWindow},
     {"updateApplicationState", "(I)V", (void *)updateApplicationState},
