@@ -121,7 +121,7 @@ QAccessibleTable::~QAccessibleTable()
 
 QHeaderView *QAccessibleTable::horizontalHeader() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
     if (false) {
 #if QT_CONFIG(tableview)
     } else if (const QTableView *tv = qobject_cast<const QTableView*>(view())) {
@@ -137,7 +137,7 @@ QHeaderView *QAccessibleTable::horizontalHeader() const
 
 QHeaderView *QAccessibleTable::verticalHeader() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
     if (false) {
 #if QT_CONFIG(tableview)
     } else if (const QTableView *tv = qobject_cast<const QTableView*>(view())) {
@@ -150,19 +150,19 @@ QHeaderView *QAccessibleTable::verticalHeader() const
 QAccessibleInterface *QAccessibleTable::cellAt(int row, int column) const
 {
     if (!view()->model())
-        return 0;
+        return nullptr;
     Q_ASSERT(role() != QAccessible::Tree);
     QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
     if (Q_UNLIKELY(!index.isValid())) {
         qWarning() << "QAccessibleTable::cellAt: invalid index: " << index << " for " << view();
-        return 0;
+        return nullptr;
     }
     return child(logicalIndex(index));
 }
 
 QAccessibleInterface *QAccessibleTable::caption() const
 {
-    return 0;
+    return nullptr;
 }
 
 QString QAccessibleTable::columnDescription(int column) const
@@ -254,7 +254,7 @@ QList<int> QAccessibleTable::selectedRows() const
 
 QAccessibleInterface *QAccessibleTable::summary() const
 {
-    return 0;
+    return nullptr;
 }
 
 bool QAccessibleTable::isColumnSelected(int column) const
@@ -422,7 +422,7 @@ QAccessibleInterface *QAccessibleTable::childAt(int x, int y) const
     if (index.isValid()) {
         return child(logicalIndex(index));
     }
-    return 0;
+    return nullptr;
 }
 
 int QAccessibleTable::childCount() const
@@ -485,13 +485,13 @@ QAccessibleInterface *QAccessibleTable::parent() const
         }
         return QAccessible::queryAccessibleInterface(view()->parent());
     }
-    return 0;
+    return nullptr;
 }
 
 QAccessibleInterface *QAccessibleTable::child(int logicalIndex) const
 {
     if (!view()->model())
-        return 0;
+        return nullptr;
 
     auto id = childToId.constFind(logicalIndex);
     if (id != childToId.constEnd())
@@ -505,7 +505,7 @@ QAccessibleInterface *QAccessibleTable::child(int logicalIndex) const
     int row = logicalIndex / columns;
     int column = logicalIndex % columns;
 
-    QAccessibleInterface *iface = 0;
+    QAccessibleInterface *iface = nullptr;
 
     if (vHeader) {
         if (column == 0) {
@@ -528,7 +528,7 @@ QAccessibleInterface *QAccessibleTable::child(int logicalIndex) const
         QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
         if (Q_UNLIKELY(!index.isValid())) {
             qWarning("QAccessibleTable::child: Invalid index at: %d %d", row, column);
-            return 0;
+            return nullptr;
         }
         iface = new QAccessibleTableCell(view(), index, cellRole());
     }
@@ -542,7 +542,7 @@ void *QAccessibleTable::interface_cast(QAccessible::InterfaceType t)
 {
     if (t == QAccessible::TableInterface)
        return static_cast<QAccessibleTableInterface*>(this);
-   return 0;
+   return nullptr;
 }
 
 void QAccessibleTable::modelChange(QAccessibleTableModelChangeEvent *event)
@@ -676,13 +676,13 @@ QModelIndex QAccessibleTree::indexFromLogical(int row, int column) const
 QAccessibleInterface *QAccessibleTree::childAt(int x, int y) const
 {
     if (!view()->model())
-        return 0;
+        return nullptr;
     QPoint viewportOffset = view()->viewport()->mapTo(view(), QPoint(0,0));
     QPoint indexPosition = view()->mapFromGlobal(QPoint(x, y) - viewportOffset);
 
     QModelIndex index = view()->indexAt(indexPosition);
     if (!index.isValid())
-        return 0;
+        return nullptr;
 
     const QTreeView *treeView = qobject_cast<const QTreeView*>(view());
     int row = treeView->d_func()->viewIndex(index) + (horizontalHeader() ? 1 : 0);
@@ -706,9 +706,9 @@ int QAccessibleTree::childCount() const
 QAccessibleInterface *QAccessibleTree::child(int logicalIndex) const
 {
     if (logicalIndex < 0 || !view()->model() || !view()->model()->columnCount())
-        return 0;
+        return nullptr;
 
-    QAccessibleInterface *iface = 0;
+    QAccessibleInterface *iface = nullptr;
     int index = logicalIndex;
 
     if (horizontalHeader()) {
@@ -724,7 +724,7 @@ QAccessibleInterface *QAccessibleTree::child(int logicalIndex) const
         int column = index % view()->model()->columnCount();
         QModelIndex modelIndex = indexFromLogical(row, column);
         if (!modelIndex.isValid())
-            return 0;
+            return nullptr;
         iface = new QAccessibleTableCell(view(), modelIndex, cellRole());
     }
     QAccessible::registerAccessibleInterface(iface);
@@ -772,7 +772,7 @@ QAccessibleInterface *QAccessibleTree::cellAt(int row, int column) const
     QModelIndex index = indexFromLogical(row, column);
     if (Q_UNLIKELY(!index.isValid())) {
         qWarning("Requested invalid tree cell: %d %d", row, column);
-        return 0;
+        return nullptr;
     }
     const QTreeView *treeView = qobject_cast<const QTreeView*>(view());
     Q_ASSERT(treeView);
@@ -841,7 +841,7 @@ void *QAccessibleTableCell::interface_cast(QAccessible::InterfaceType t)
         return static_cast<QAccessibleTableCellInterface*>(this);
     if (t == QAccessible::ActionInterface)
         return static_cast<QAccessibleActionInterface*>(this);
-    return 0;
+    return nullptr;
 }
 
 int QAccessibleTableCell::columnExtent() const { return 1; }
@@ -869,7 +869,7 @@ QList<QAccessibleInterface*> QAccessibleTableCell::columnHeaderCells() const
 
 QHeaderView *QAccessibleTableCell::horizontalHeader() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
 
     if (false) {
 #if QT_CONFIG(tableview)
@@ -887,7 +887,7 @@ QHeaderView *QAccessibleTableCell::horizontalHeader() const
 
 QHeaderView *QAccessibleTableCell::verticalHeader() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
 #if QT_CONFIG(tableview)
     if (const QTableView *tv = qobject_cast<const QTableView*>(view))
         header = tv->verticalHeader();
@@ -934,10 +934,23 @@ QStringList QAccessibleTableCell::actionNames() const
 void QAccessibleTableCell::doAction(const QString& actionName)
 {
     if (actionName == toggleAction()) {
-        if (isSelected())
+#if defined(Q_OS_ANDROID)
+        QAccessibleInterface *parentInterface = parent();
+        while (parentInterface){
+            if (parentInterface->role() == QAccessible::ComboBox) {
+                selectCell();
+                parentInterface->actionInterface()->doAction(pressAction());
+                return;
+            } else {
+                parentInterface = parentInterface->parent();
+            }
+        }
+#endif
+        if (isSelected()) {
             unselectCell();
-        else
+        } else {
             selectCell();
+        }
     }
 }
 
@@ -1035,10 +1048,14 @@ QAccessible::State QAccessibleTableCell::state() const
         st.selected = true;
     if (view->selectionModel()->currentIndex() == m_index)
         st.focused = true;
-    if (m_index.model()->data(m_index, Qt::CheckStateRole).toInt() == Qt::Checked)
+
+    QVariant checkState = m_index.model()->data(m_index, Qt::CheckStateRole);
+    if (checkState.toInt() == Qt::Checked)
         st.checked = true;
 
     Qt::ItemFlags flags = m_index.flags();
+    if ((flags & Qt::ItemIsUserCheckable) && checkState.isValid())
+        st.checkable = true;
     if (flags & Qt::ItemIsSelectable) {
         st.selectable = true;
         st.focusable = true;
@@ -1115,7 +1132,7 @@ QAccessibleInterface *QAccessibleTableCell::parent() const
 
 QAccessibleInterface *QAccessibleTableCell::child(int) const
 {
-    return 0;
+    return nullptr;
 }
 
 QAccessibleTableHeaderCell::QAccessibleTableHeaderCell(QAbstractItemView *view_, int index_, Qt::Orientation orientation_)
@@ -1143,7 +1160,7 @@ QAccessible::State QAccessibleTableHeaderCell::state() const
 
 QRect QAccessibleTableHeaderCell::rect() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
     if (false) {
 #if QT_CONFIG(tableview)
     } else if (const QTableView *tv = qobject_cast<const QTableView*>(view)) {
@@ -1206,12 +1223,12 @@ QAccessibleInterface *QAccessibleTableHeaderCell::parent() const
 
 QAccessibleInterface *QAccessibleTableHeaderCell::child(int) const
 {
-    return 0;
+    return nullptr;
 }
 
 QHeaderView *QAccessibleTableHeaderCell::headerView() const
 {
-    QHeaderView *header = 0;
+    QHeaderView *header = nullptr;
     if (false) {
 #if QT_CONFIG(tableview)
     } else if (const QTableView *tv = qobject_cast<const QTableView*>(view)) {
