@@ -76,6 +76,9 @@
 #include <qlineedit.h>
 #include <private/qlineedit_p.h>
 #endif
+#ifndef QT_NO_PICTURE
+#include <QtGui/qpicture.h>
+#endif
 #include <qstyle.h>
 #include <qstyleoption.h>
 #include <qtextdocument.h>
@@ -364,7 +367,7 @@ QAccessibleInterface *QAccessibleToolButton::child(int index) const
 #else
     Q_UNUSED(index)
 #endif
-    return 0;
+    return nullptr;
 }
 
 /*
@@ -431,10 +434,10 @@ QAccessible::Role QAccessibleDisplay::role() const
 #if QT_CONFIG(label)
     QLabel *l = qobject_cast<QLabel*>(object());
     if (l) {
-        if (l->pixmap())
+        if (!l->pixmap(Qt::ReturnByValue).isNull())
             return QAccessible::Graphic;
 #ifndef QT_NO_PICTURE
-        if (l->picture())
+        if (!l->picture(Qt::ReturnByValue).isNull())
             return QAccessible::Graphic;
 #endif
 #if QT_CONFIG(movie)
@@ -558,10 +561,7 @@ QSize QAccessibleDisplay::imageSize() const
 #endif
         return QSize();
 #if QT_CONFIG(label)
-    const QPixmap *pixmap = label->pixmap();
-    if (!pixmap)
-        return QSize();
-    return pixmap->size();
+    return label->pixmap(Qt::ReturnByValue).size();
 #endif
 }
 
@@ -574,8 +574,7 @@ QPoint QAccessibleDisplay::imagePosition() const
 #endif
         return QPoint();
 #if QT_CONFIG(label)
-    const QPixmap *pixmap = label->pixmap();
-    if (!pixmap)
+    if (label->pixmap(Qt::ReturnByValue).isNull())
         return QPoint();
 
     return QPoint(label->mapToGlobal(label->pos()));
@@ -779,8 +778,7 @@ int QAccessibleLineEdit::cursorPosition() const
 QRect QAccessibleLineEdit::characterRect(int offset) const
 {
     int x = lineEdit()->d_func()->control->cursorToX(offset);
-    int y;
-    lineEdit()->getTextMargins(0, &y, 0, 0);
+    int y = lineEdit()->textMargins().top();
     QFontMetrics fm(lineEdit()->font());
     const QString ch = text(offset, offset + 1);
     if (ch.isEmpty())
@@ -977,7 +975,7 @@ QAccessibleInterface *QAccessibleWindowContainer::child(int i) const
 {
     if (i == 0)
         return QAccessible::queryAccessibleInterface(container()->containedWindow());
-    return 0;
+    return nullptr;
 }
 
 QWindowContainer *QAccessibleWindowContainer::container() const
